#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

/* Program demonstrira fork sistemski poziv, tj. nacin za kreiranje
   novog procesa na Unix sistemima.*/
int main(){ 
  char write_buffer[] = "Poruka iz write.\n"; 
  char printf_buffer[] = "Poruka iz printf.\n";
  pid_t pid;

  /* Sistemski poziv write nije baferovan, pa se poruka momentalno ispisuje. */
  if (write (STDOUT_FILENO, write_buffer, strlen (write_buffer)) != strlen (write_buffer))
    error_fatal ("write error of write_buffer");

  /* Funkcija printf je linijski baferovana, pa se poruka momentalno
     ispisuje jer sadrzi \n. Ako se izlaz preusmeri u fajl, posto se
     za fajlove koristi puno baferisanje, sadrzaji bafera bice
     iskopirani u dete proces. To znaci da ce u tom slucaju poruka
     biti ispisana dva puta: jednom u roditelju, jednom u detetu. */
  printf ("%s",printf_buffer);

  /* Sistemski poziv fork kreira novi proces. */
  if ( (pid = fork ()) < 0)
    error_fatal ("fork error");
  /* Ako je vracen broj 0, onda znamo da se nalazimo u detetu procesu. */
  else if (pid == 0)
    printf ("Poruka iz deteta, id: %d\n", getpid());
  /* Inace se nalazimo u roditeljskom procesu. */
  else {
    sleep (2); // Cekanje da se dete proces prvo zavrsi
    printf ("Poruka iz roditelja, id: %d.\n", getpid());
  }

  printf ("Ovu poruku ispisuju i roditelj i dete.\n");
  return 0;
}

