#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

/* Program demonstrira kako init proces nasledjuje proces C koji se
   zavrsio POSLE zavrsetka svog roditeljskog procesa B. Takodje
   demonstrira da vise fork poziva mogu da budu ugnjezdeni. Proces A
   kreira dete proces B. Proces B kreira dete proces C.  A ceka na B i
   B zavrsava sa radom. Proces C biva nasledjen od init procesa,
   tj. proces ID njegovog roditelja je 0. Posto A doceka kraj B, odmah
   se zavrsava. To znaci da ce C zadnji od svih da se zavrsi,
   tj. kontrola ce vec biti vracena shell-u kada C ispise poruku. */
int main(){
  pid_t pid;

  // A kreira dete proces B.
  if ( (pid = fork ()) < 0)
    error_fatal ("fork error");

  // U procesu B.
  else if (pid == 0){

    // B kreira dete proces C.
    if ( (pid = fork()) < 0)
      error_fatal ("fork error");
    // C ceka 3 sekunde da se B zavrsi.
    else if (pid == 0){
      sleep (3);
      printf ("Parent process id: %d\n", getppid());
    }
    // B se momentalno zavrsava
    else 
      exit (EXIT_SUCCESS);
  }
  
  // U prosesu A.
  else {
    // A ceka na B
    if (waitpid (pid, NULL, 0) < 0)
      error_fatal ("waitpid error");
  }

  return 0;
}

