#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/* Funkcija status_print() stampa informacije o zavrsnom statusu procesa. */
static void status_print (int status);

static void error_fatal (char *message);

/* Program kreira dete proces u kome menja odredjene promjenljive, a potom
 * oba procesa stampaju te vrijednosti.
 */
int
main (int argc, char **argv)
{
  pid_t pid;			/* Identifikator procesa. */
  int status;			/* Informacija o zavrsetku dete procesa. */

  /* Kreira se dete proces koji se normalno zavrsava i stampaju se
     podaci o njegovom zavrsnom statusu.*/
  if ((pid = fork ()) < 0)
    error_fatal (argv[0]);
  else if (pid == 0)
    exit (42);

  if (wait (&status) != pid)
    error_fatal (argv[0]);
  status_print (status);

  /* Kreira se dete proces koji se nasilno prekida i stampaju se
     podaci o njegovom zavrsnom statusu.*/
  if ((pid = fork ()) < 0)
    error_fatal (argv[0]);
  else if (pid == 0)
    abort ();
  if (wait (&status) != pid)
    error_fatal (argv[0]);
  status_print (status);

  /* Kreira se dete proces u kome se deli nulom i stampaju
     se podaci o njegovom zavrsnom statusu.*/
  if ((pid = fork ()) < 0)
    error_fatal (argv[0]);
  else if (pid == 0)
    status /= 0;
  if (wait (&status) != pid)
    error_fatal (argv[0]);
  status_print (status);

  /* Zavrsava se program. */
  exit (EXIT_SUCCESS);
}

static void status_print (int status){
  /* U skladu sa zavrsnim statusom procesa, stampa se odgovarajuca
     poruka. Makroi za proveru statusa su opisani u man stranici
     funkcije wait.*/
  if (WIFEXITED (status))
    printf ("normal termination, exit status = %d\n", WEXITSTATUS (status));
  else if (WIFSIGNALED (status))
    printf ("abnormal termination, signal number = %d\n", WTERMSIG (status));
}

static void error_fatal (char *message){
  perror (message);
  exit (EXIT_FAILURE);
}
